/**************************************************************************//**
 * @file     main.c
 * @version  V1.00
 * @brief    NUC230_240 Series control DS3231 by I2C interface
 *
 * @note
 * Copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NUC230_240.h"


/*----------------------------------------------------------------------------*/
/* Define                                                                     */
/*----------------------------------------------------------------------------*/
#define Seconds 0x00
#define Miutes  0x01
#define Hours   0x02
#define Day     0x03
#define Date    0x04
#define Month   0x05
#define Year    0x06


/*----------------------------------------------------------------------------*/
/* Global variables                                                           */
/*----------------------------------------------------------------------------*/
volatile uint8_t g_u8DeviceAddr;
volatile uint8_t g_au8TxData[3];
volatile uint8_t g_u8RxData;
volatile uint8_t g_u8DataLen;
volatile uint8_t g_u8EndFlag = 0;

typedef void (*I2C_FUNC)(uint32_t u32Status);
static I2C_FUNC s_I2C0HandlerFn = NULL;


/*----------------------------------------------------------------------------*/
/* Functions                                                                  */
/*----------------------------------------------------------------------------*/


/**
 * @brief       I2C0 IRQ
 *
 * @param       None
 *
 * @return      None
 *
 * @details     The I2C0 default IRQ, declared in startup_NUC230_240.s.
 */
void I2C0_IRQHandler(void)
{
    uint32_t u32Status;
    u32Status = I2C_GET_STATUS(I2C0);

    if (I2C_GET_TIMEOUT_FLAG(I2C0))
    {
        /* Clear I2C0 Timeout Flag */
        I2C_ClearTimeoutFlag(I2C0);
    }
    else
    {
        if (s_I2C0HandlerFn != NULL)
        {
            s_I2C0HandlerFn(u32Status);
        }
    }
}


/*----------------------------------------------------------------------------*/
/*  I2C Rx Callback Function                                                  */
/*----------------------------------------------------------------------------*/
void I2C_MasterRx(uint32_t u32Status)
{
    /* START has been transmitted and prepare SLA+W */
    if (u32Status == 0x08)
    {
        /* Write SLA+W to Register I2CDAT */
        I2C_SET_DATA(I2C0, (g_u8DeviceAddr << 1));
        I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_SI);
    }
    /* SLA+W has been transmitted and ACK has been received */
    else if (u32Status == 0x18)
    {
        I2C_SET_DATA(I2C0, g_au8TxData[g_u8DataLen++]);
        I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_SI);
    }
    /* SLA+W has been transmitted and NACK has been received */
    else if (u32Status == 0x20)
    {
        I2C_STOP(I2C0);
        I2C_START(I2C0);
    }
    /* DATA has been transmitted and ACK has been received */
    else if (u32Status == 0x28)
    {
        if (g_u8DataLen != 1)
        {
            I2C_SET_DATA(I2C0, g_au8TxData[g_u8DataLen++]);
            I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_SI);
        }
        else
        {
            I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_STA_SI);
        }
    }
    /* Repeat START has been transmitted and prepare SLA+R */
    else if (u32Status == 0x10)
    {
        /* Write SLA+R to Register I2CDAT */
        I2C_SET_DATA(I2C0, ((g_u8DeviceAddr << 1) | 0x01));
        I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_SI);
    }
    /* SLA+R has been transmitted and ACK has been received */
    else if (u32Status == 0x40)
    {
        I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_SI);
    }
    /* DATA has been received and NACK has been returned */
    else if (u32Status == 0x58)
    {
        g_u8RxData = (unsigned char) I2C_GET_DATA(I2C0);
        I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_STO_SI);
        g_u8EndFlag = 1;
    }
    else
    {
        /* TO DO */
        printf("Status 0x%x is NOT processed\n", u32Status);
    }
}


/*----------------------------------------------------------------------------*/
/*  I2C Tx Callback Function                                                  */
/*----------------------------------------------------------------------------*/
void I2C_MasterTx(uint32_t u32Status)
{
    /* START has been transmitted */
    if (u32Status == 0x08)
    {
        /* Write SLA+W to Register I2CDAT */
        I2C_SET_DATA(I2C0, g_u8DeviceAddr << 1);
        I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_SI);
    }
    /* SLA+W has been transmitted and ACK has been received */
    else if (u32Status == 0x18)
    {
        I2C_SET_DATA(I2C0, g_au8TxData[g_u8DataLen++]);
        I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_SI);
    }
    /* SLA+W has been transmitted and NACK has been received */
    else if (u32Status == 0x20)
    {
        I2C_STOP(I2C0);
        I2C_START(I2C0);
    }
    /* DATA has been transmitted and ACK has been received */
    else if (u32Status == 0x28)
    {
        if (g_u8DataLen != 2)
        {
            I2C_SET_DATA(I2C0, g_au8TxData[g_u8DataLen++]);
            I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_SI);
        }
        else
        {
            I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_STO_SI);
            g_u8EndFlag = 1;
        }
    }
    else
    {
        /* TO DO */
        printf("Status 0x%x is NOT processed\n", u32Status);
    }
}


void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();
    /* Enable Internal RC 22.1184MHz clock */
    CLK_EnableXtalRC(CLK_PWRCON_OSC22M_EN_Msk);
    /* Waiting for Internal RC clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_OSC22M_STB_Msk);
    /* Switch HCLK clock source to Internal RC and HCLK source divide 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_CLKDIV_HCLK(1));
    /* Enable external XTAL 12MHz clock */
    CLK_EnableXtalRC(CLK_PWRCON_XTL12M_EN_Msk);
    /* Waiting for external XTAL clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_XTL12M_STB_Msk);
    /* Enable UART module clock */
    CLK_EnableModuleClock(UART0_MODULE);
    /* Enable I2C0 module clock */
    CLK_EnableModuleClock(I2C0_MODULE);
    /* Select UART module clock source */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART_S_HXT, CLK_CLKDIV_UART(1));
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set GPB multi-function pins for UART0 RXD and TXD */
    SYS->GPB_MFP = SYS_GPB_MFP_PB0_UART0_RXD | SYS_GPB_MFP_PB1_UART0_TXD;
    /* Set GPA multi-function pins for I2C0 SDA and SCL */
    SYS->GPA_MFP = SYS_GPA_MFP_PA8_I2C0_SDA | SYS_GPA_MFP_PA9_I2C0_SCL;
    /* Lock protected registers */
    SYS_LockReg();
}


void UART0_Init(void)
{
    /* Reset IP */
    SYS_ResetModule(UART0_RST);
    /* Configure UART0 and set UART0 Baudrate */
    UART_Open(UART0, 115200);
}


void I2C0_Close(void)
{
    /* Disable I2C0 interrupt and clear corresponding NVIC bit */
    I2C_DisableInt(I2C0);
    NVIC_DisableIRQ(I2C0_IRQn);
    /* Disable I2C0 and close I2C0 clock */
    I2C_Close(I2C0);
    CLK_DisableModuleClock(I2C0_MODULE);
}


void I2C0_Init(void)
{
    /* Open I2C module and set bus clock */
    I2C_Open(I2C0, 100000);
    /* Get I2C0 Bus Clock */
    printf("I2C clock %d Hz\n", I2C_GetBusClockFreq(I2C0));
    /* Enable I2C interrupt */
    I2C_EnableInt(I2C0);
    NVIC_EnableIRQ(I2C0_IRQn);
}


uint8_t dec_to_bcd(uint32_t u32Dec)
{
    uint8_t uBcd = ((u32Dec / 10) << 4) | (u32Dec % 10);
    return uBcd;
}


uint32_t bcd_to_dec(uint8_t u8Hex)
{
    uint32_t uDec = ((u8Hex & 0xF0) >> 4) * 10 + (u8Hex & 0x0F);
    return uDec;
}


uint8_t DS3231_RTC_Read(uint8_t u8Reg)
{
    /* DS3231 I2C slave address */
    g_u8DeviceAddr = 0x68;
    /* DS3231 address map */
    g_au8TxData[0] = u8Reg;
    g_u8DataLen = 0;
    g_u8EndFlag = 0;
    /* I2C function to read data from slave */
    s_I2C0HandlerFn = (I2C_FUNC)I2C_MasterRx;
    /* I2C as master sends START signal */
    I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_STA);

    /* Wait I2C Rx Finish */
    while (g_u8EndFlag == 0);

    g_u8EndFlag = 0;
    return bcd_to_dec(g_u8RxData);
}


void DS3231_RTC_Write(uint8_t u8Reg, uint8_t u8Value)
{
    /* DS3231 I2C slave address */
    g_u8DeviceAddr = 0x68;
    /* DS3231 address map */
    g_au8TxData[0] = u8Reg;
    /* Write data to DS3231 */
    g_au8TxData[1] = dec_to_bcd(u8Value);
    g_u8DataLen = 0;
    g_u8EndFlag = 0;
    /* I2C function to write data to slave */
    s_I2C0HandlerFn = (I2C_FUNC)I2C_MasterTx;
    /* I2C as master sends START signal */
    I2C_SET_CONTROL_REG(I2C0, I2C_I2CON_STA);

    /* Wait I2C Tx Finish */
    while (g_u8EndFlag == 0);

    g_u8EndFlag = 0;
}


/*----------------------------------------------------------------------------*/
/* MAIN function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
    uint8_t u8SecondsTemp;
    /* Init System, IP clock and multi-function I/O */
    SYS_Init();
    /* Init UART0 to 115200-8n1 for print message */
    UART0_Init();
    /* Init I2C0 */
    I2C0_Init();
    /* Init DS3231 */
    DS3231_RTC_Write(Hours,  15);
    DS3231_RTC_Write(Miutes, 57);
    DS3231_RTC_Write(Seconds, 57);
    DS3231_RTC_Write(Day,     2);
    DS3231_RTC_Write(Date,    5);
    DS3231_RTC_Write(Month,  11);
    DS3231_RTC_Write(Year,   19);
    u8SecondsTemp = 0;

    while (1)
    {
        if (u8SecondsTemp != DS3231_RTC_Read(Seconds))
        {
            u8SecondsTemp = DS3231_RTC_Read(Seconds);
            printf("%d/", DS3231_RTC_Read(Year));
            printf("%d/", DS3231_RTC_Read(Month));
            printf("%d ", DS3231_RTC_Read(Date));
            printf("%d: ", DS3231_RTC_Read(Hours));
            printf("%d: ", DS3231_RTC_Read(Miutes));
            printf("%02d\n\r", u8SecondsTemp);
        }
    }
}

/*** (C) COPYRIGHT 2019 Nuvoton Technology Corp. ***/
